@@ -111,6 +111,25 @@ namespace OpenBabel
     obLocale.RestoreLocale();
   }
  
+  vector3 GetCorrectedBondVector(OBAtom *atom1, OBAtom *atom2, int bondOrder = 1)
+  {
+    double bondLength = 0.0;
+
+    // We create an estimate of the bond length based on the two atoms
+    bondLength += etab.CorrectedBondRad(atom1->GetAtomicNum(), atom1->GetHyb());
+    bondLength += etab.CorrectedBondRad(atom2->GetAtomicNum(), atom2->GetHyb());
+
+    // These are based on OBBond::GetEquibLength
+    if (bondOrder == -1) // aromatic
+      bondLength *= 0.93;
+    else if (bondOrder == 2)
+      bondLength *= 0.91;
+    else if (bondOrder == 3)
+      bondLength *= 0.87;
+
+    return OBBuilder::GetNewBondVector(atom1, bondLength);
+  }
+
   vector3 OBBuilder::GetNewBondVector(OBAtom *atom)
   {
     return GetNewBondVector(atom, 1.5);
@@ -160,7 +179,10 @@ namespace OpenBabel
     //                               *
     if (atom->GetValence() == 1) {
       FOR_NBORS_OF_ATOM (nbr, atom) {
-        bond1 = atom->GetVector() - nbr->GetVector();   
+        bond1 = atom->GetVector() - nbr->GetVector();
+
+        if (nbr->GetHyb() == 1) // Fix #2119034 & #2013814
+          continue;
         
         FOR_NBORS_OF_ATOM (nbr2, &*nbr)
           if (&*nbr2 != atom)
@@ -268,7 +290,9 @@ namespace OpenBabel
   // fragment and the fragment itself. The fragment containing b will be 
   // rotated and translated. Atom a is the atom from 
   // the main molecule to which we want to connect atom b.
-  bool OBBuilder::Connect(OBMol &mol, int idxA, int idxB, vector3 &newpos, int bondOrder)
+  // NOTE: newpos now uses CorrectedBondVector, so we don't do that below
+  bool OBBuilder::Connect(OBMol &mol, int idxA, int idxB, 
+                          vector3 &newpos, int bondOrder)
   {
     OBAtom *a = mol.GetAtom(idxA);
     OBAtom *b = mol.GetAtom(idxB);
@@ -381,7 +405,7 @@ namespace OpenBabel
 
   bool OBBuilder::Connect(OBMol &mol, int idxA, int idxB, int bondOrder)
   {
-    vector3 newpos = GetNewBondVector(mol.GetAtom(idxA));
+    vector3 newpos = GetCorrectedBondVector(mol.GetAtom(idxA), mol.GetAtom(idxB), bondOrder);
     return Connect(mol, idxA, idxB, newpos, bondOrder);
   }
 
@@ -499,6 +523,7 @@ namespace OpenBabel
     OBMol workMol = mol;
     
     // delete all bonds in the working molecule
+    // we will add them back at the end
     while (workMol.NumBonds())
       workMol.DeleteBond(workMol.GetBond(0));
     
@@ -532,7 +557,7 @@ namespace OpenBabel
             atom->SetVector(i->second[counter]);
             counter++;
           }
-            
+
           // add the bonds for the fragment
           for (k = j->begin(); k != j->end(); ++k) {
             index = *k;
@@ -583,7 +608,13 @@ namespace OpenBabel
    
       // get the position for the new atom, this is done with GetNewBondVector
       if (prev != NULL) {
-        molvec = GetNewBondVector(workMol.GetAtom(prev->GetIdx()));
+        int bondType = a->GetBond(prev)->GetBO();
+        if (a->GetBond(prev)->IsAromatic())
+          bondType = -1;
+
+        molvec = GetCorrectedBondVector(workMol.GetAtom(prev->GetIdx()),
+                                        workMol.GetAtom(a->GetIdx()),
+                                        bondType);
         moldir = molvec - workMol.GetAtom(prev->GetIdx())->GetVector();
       } else {
         molvec = VX;
@@ -603,6 +634,17 @@ namespace OpenBabel
       }
 
     }
+
+    // Ensure all bonds from the old molecule exist in the new molecule
+    int beginIdx, endIdx;
+    FOR_BONDS_OF_MOL(b, mol) {
+      beginIdx = b->GetBeginAtomIdx();
+      endIdx = b->GetEndAtomIdx();
+      if (!workMol.GetBond(beginIdx, endIdx)) {
+        // We need to duplicate the old bond
+        workMol.AddBond(beginIdx, endIdx, b->GetBO(), b->GetFlags());
+      }
+    }
     
     // correct the chirality
     CorrectStereoBonds(workMol);
