@@ -48,7 +48,9 @@ using namespace std;
 //! An index of the residue names perceived during a run
 //! 0, 1, and 2 reserved for UNK, HOH, LIG
 static char ChainsResName[RESIDMAX][4] = {
-  /*0*/ "UNK",  /*1*/ "HOH",  /*2*/ "LIG"
+  /*0*/ "UNK",  
+  /*1*/ "HOH",  
+  /*2*/ "LIG"
 };
 
 #define ATOMMINAMINO   4
@@ -200,7 +202,7 @@ namespace OpenBabel
   //! PDB atom types (i.e., columns 13-16 of a PDB file)
   //!  index numbers from this array are used in the pseudo-SMILES format
   //!  for side-chains in the AminoAcids[] & Nucleotides[] global arrays below
-  static const char ChainsAtomName[ATOMMAX][4] = {
+  static char ChainsAtomName[ATOMMAX][4] = {
     /*  0 */  { ' ', 'N', ' ', ' ' },
     /*  1 */  { ' ', 'C', 'A', ' ' },
     /*  2 */  { ' ', 'C', ' ', ' ' },
@@ -283,7 +285,7 @@ namespace OpenBabel
   //! Side chains for recognized amino acids using a pseudo-SMARTS syntax
   //!  for branching and bonds. Numbers indicate atom types defined by 
   //!  ChainsAtomName global array above
-  static const ResidType AminoAcids[AMINOMAX] = {
+  static ResidType AminoAcids[AMINOMAX] = {
     { "ILE", "1-4(-9-14)-10"                        },
     { "VAL", "1-4(-9)-10"                           },
     { "ALA", "1-4"                                  },
@@ -314,7 +316,7 @@ namespace OpenBabel
   //! Side chains for recognized nucleotides using a pseudo-SMARTS syntax
   //!  for branching and bonds. Numbers indicate atom types defined by 
   //!  ChainsAtomName global array above
-  static const ResidType Nucleotides[NUCLEOMAX] = {
+  static ResidType Nucleotides[NUCLEOMAX] = {
     { "  A", "49-50-51-52-53-54(-56)-57-58-61-62(-53)-50"      },
     { "  C", "49-57-58(-59)-61-62(-64)-65-67-57"               },
     { "  G", "49-50-51-52-53-54(-55)-57-58(-60)-61-62(-53)-50" },
@@ -405,11 +407,10 @@ namespace OpenBabel
   {
     ByteCode *result;
 
-    result = (ByteCode*)malloc(sizeof(ByteCode));
+    result = new ByteCode;
     if( !result )
       {
         obErrorLog.ThrowError(__FUNCTION__, "Unable to allocate byte codes for biomolecule residue perception.", obError);
-        //        exit(1);
       }
     result->type = type;
     result->eval.next     = NULL;
@@ -438,10 +439,14 @@ namespace OpenBabel
           {
           case BC_ASSIGN:
 
-            if (node->assign.atomid != NULL)
-              free(node->assign.atomid);
-            if (node->assign.bflags != NULL)
-              free(node->assign.bflags);
+            if (node->assign.atomid != NULL) {
+	      delete [] node->assign.atomid;
+              node->assign.atomid = NULL; // prevent double-free
+            }
+            if (node->assign.bflags != NULL) {
+              delete [] node->assign.bflags;
+              node->assign.bflags = NULL; // prevent double-free
+            }
             
             break;
 
@@ -474,14 +479,14 @@ namespace OpenBabel
             break;
           }
 
-        free(node);
+	delete node;
+        node = NULL;
       }
   }
 
   static void FatalMemoryError(void)
   {
     obErrorLog.ThrowError(__FUNCTION__, "Unable to allocate memory for biomolecule residue / chain perception.", obError);
-    //    exit(1);
   }
 
   void GenerateByteCodes(ByteCode **node, int resid, int curr, int prev, int bond)
@@ -697,7 +702,7 @@ namespace OpenBabel
       {
         ptr = AllocateByteCode(BC_ASSIGN);
         ptr->assign.resid = resid;
-        ptr->assign.atomid = (int*)malloc(AtomIndex*sizeof(int));
+        ptr->assign.atomid = new int[AtomIndex];
         if( !ptr->assign.atomid )
           FatalMemoryError();
         for( i=0; i<MonoAtomCount; i++ )
@@ -705,7 +710,7 @@ namespace OpenBabel
             ptr->assign.atomid[j] = MonoAtom[i].atomid;
         if( BondIndex )
           {
-            ptr->assign.bflags = (int*)malloc(BondIndex*sizeof(int));
+            ptr->assign.bflags = new int[BondIndex];
             for( i=0; i<MonoBondCount; i++ )
               if( (j=MonoBond[i].index) != -1 )
                 ptr->assign.bflags[j] = MonoBond[i].flag;
@@ -743,10 +748,7 @@ namespace OpenBabel
   //////////////////////////////////////////////////////////////////////////////
 
   // validated
-  OBChainsParser::OBChainsParser(void) :
-    bitmasks(NULL), visits(NULL), resids  (NULL), flags   (NULL),
-    hetflags(NULL), atomids (NULL), resnos  (NULL), sernos  (NULL),
-    hcounts (NULL), chains  (NULL)
+  OBChainsParser::OBChainsParser(void)
   {
     int i, res = RESIDMIN;
 
@@ -788,27 +790,16 @@ namespace OpenBabel
     int asize = mol.NumAtoms();
     int bsize = mol.NumBonds();
 
-    bitmasks = new unsigned short[asize];
-    visits   = new bool[asize];
-    resids   = new unsigned char[asize];
-    flags    = new unsigned char[bsize];
-    hetflags = new bool[asize];
-    atomids  = new int[asize];
-    resnos   = new short[asize];
-    sernos   = new short[asize];
-    hcounts  = new char[asize];
-    chains   = new char[asize];
-
-    memset(bitmasks, 0,   sizeof(unsigned short) * asize);
-    memset(visits,   0,   sizeof(bool)           * asize);
-    memset(resids,   0,   sizeof(unsigned char)  * asize);
-    memset(hetflags, 0,   sizeof(bool)           * asize);
-    memset(resnos,   0,   sizeof(short)          * asize);
-    memset(sernos,   0,   sizeof(short)          * asize);
-    memset(hcounts,  0,   sizeof(char)           * asize);
-    memset(chains,   ' ', sizeof(char)           * asize);
-    
-    memset(flags,    0,   sizeof(unsigned char)  * bsize);
+    bitmasks.resize(asize, 0);
+    visits.resize(asize, 0);
+    resids.resize(asize, 0);
+    flags.resize(bsize, 0);
+    hetflags.resize(asize, 0);
+    atomids.resize(asize, 0);
+    resnos.resize(asize, 0);
+    sernos.resize(asize, 0);
+    hcounts.resize(asize, 0);
+    chains.resize(asize, ' ');
 
     for ( i = 0 ; i < asize ; i++ )
       {
@@ -820,56 +811,16 @@ namespace OpenBabel
   //! Used by OBChainsParser::SetupMol()
   void OBChainsParser::CleanupMol(void)
   {
-    if (bitmasks != NULL)
-      {
-        delete bitmasks;
-        bitmasks = NULL;
-      }
-    if (visits != NULL)
-      {
-        delete visits;
-        visits = NULL;
-      }
-    if (hetflags != NULL)
-      {
-        delete hetflags;
-        hetflags = NULL;
-      }
-    if (atomids  != NULL)
-      {
-        delete atomids;
-        atomids = NULL;
-      }
-    if (resids   != NULL)
-      {
-        delete resids;
-        resids = NULL;
-      }
-    if (resnos   != NULL)
-      {
-        delete resnos;
-        resnos = NULL;
-      }
-    if (sernos   != NULL)
-      {
-        delete sernos;
-        sernos = NULL;
-      }
-    if (hcounts  != NULL)
-      {
-        delete hcounts;
-        hcounts = NULL;
-      }
-    if (chains   != NULL)
-      {
-        delete chains;
-        chains = NULL;
-      }
-    if (flags    != NULL)
-      {
-        delete flags;
-        flags = NULL;
-      }
+    bitmasks.clear();
+    visits.clear();
+    resids.clear();
+    flags.clear();
+    hetflags.clear();
+    atomids.clear();
+    resnos.clear();
+    sernos.clear();
+    hcounts.clear();
+    chains.clear();
   }
 
   //! Clear all residue information for a supplied molecule
@@ -879,6 +830,9 @@ namespace OpenBabel
     vector<OBResidue*> residues;
     vector<OBResidue*>::iterator r;
 
+    if (mol.NumResidues() == 0)
+      return; // Done!
+
     for (residue = mol.BeginResidue(r) ; residue ; residue = mol.NextResidue(r))
       residues.push_back(residue);
 
@@ -896,77 +850,82 @@ namespace OpenBabel
 
     OBAtom    *atom;
     OBResidue *residue;
-    map<short, OBResidue *> resmap;
+    map<char, map<short, OBResidue*> > resmap;
+    unsigned int numAtoms = mol.NumAtoms();
 
-    int size = mol.NumAtoms();
-    for ( int i = 0 ; i < size ; i++ )
-      {
-        atom = mol.GetAtom(i+1); // WARNING: ATOM INDEX ISSUE
+    //DumpState();
+    
+    // correct serine OG
+    for ( unsigned int i = 0 ; i < numAtoms ; i++ ) {
+      if (resids[i] == RESIDMIN + 17) // serine
+	if (atomids[i] == -1) {
+	  atom = mol.GetAtom(i+1);
 
-        if (atomids[i] == -1)
-          {
-            symbol = etab.GetSymbol(atom->GetAtomicNum());
-            if( symbol[1] )
-              {
-                buffer[0] = symbol[0];
-                buffer[1] = (char) toupper(symbol[1]);
-              }
-            else
-              {
-                buffer[0] = ' ';
-                buffer[1] = symbol[0];
-              }
-            buffer[2] = ' ';
-            buffer[3] = ' ';
-            buffer[4] = '\0';
-          }
-        else if (atom->IsHydrogen())
-          {
-            if (hcounts[i])
-              sprintf(buffer, "%cH%.2s", hcounts[i]+'0', ChainsAtomName[atomids[i]]+2);
-            else
-              sprintf(buffer, "H%.2s", ChainsAtomName[atomids[i]]+2);
-          }
-        else
-          sprintf(buffer, "%.4s", ChainsAtomName[atomids[i]]);
+	  FOR_NBORS_OF_ATOM (nbr, atom) {
+	    if (atomids[nbr->GetIdx()-1] == 4) // CB
+              atomids[i] = 6; // OG
+	  }
+	}
+    }
 
-        if (buffer[3] == ' ')
-          buffer[3] = '\0';
+    for ( unsigned int i = 0 ; i < numAtoms ; i++ ) {
+      atom = mol.GetAtom(i+1); // WARNING: ATOM INDEX ISSUE
 
-        atomid = (buffer[0] == ' ') ? buffer + 1 : buffer;
+      if (atomids[i] == -1) {
+        symbol = etab.GetSymbol(atom->GetAtomicNum());
+        if ( symbol[1] ) {
+          buffer[0] = symbol[0];
+          buffer[1] = (char) toupper(symbol[1]);
+        } else {
+          buffer[0] = ' ';
+          buffer[1] = symbol[0];
+        }
+        buffer[2] = ' ';
+        buffer[3] = ' ';
+        buffer[4] = '\0';
+      } else if (atom->IsHydrogen()) {
+        if (hcounts[i]) {
+          snprintf(buffer, BUFF_SIZE, "H%.2s%c", ChainsAtomName[atomids[i]]+2, hcounts[i]+'0');
+	  if (buffer[2] == ' ') {
+            buffer[2] = buffer[3];
+            buffer[3] = '\0';
+	  }
+	} else
+          snprintf(buffer, BUFF_SIZE, "H%.2s", ChainsAtomName[atomids[i]]+2);
+      } else
+        snprintf(buffer, BUFF_SIZE, "%.4s", ChainsAtomName[atomids[i]]);
 
-        if (resmap.find(resnos[i]) != resmap.end())
-          {
-            residue = resmap[resnos[i]];
-            residue->AddAtom(atom);
-            residue->SetAtomID(atom, atomid);
-            residue->SetHetAtom(atom, hetflags[i]);
-            residue->SetSerialNum(atom, sernos[i]);
-          }
-        else
-          {
-            name    = ChainsResName[resids[i]];
+      if (buffer[3] == ' ')
+        buffer[3] = '\0';
 
-            residue = mol.NewResidue();
+      atomid = (buffer[0] == ' ') ? buffer + 1 : buffer;
 
-            residue->SetName(name);
-            residue->SetNum(resnos[i]);
-            residue->SetChain(chains[i]);
-            residue->SetChainNum((chains[i] > 'A') ? (int)(chains[i] - 'A') : 1);
+      if (resmap[chains[i]].find(resnos[i]) != resmap[chains[i]].end()) {
+        residue = resmap[chains[i]][resnos[i]];
+        residue->AddAtom(atom);
+        residue->SetAtomID(atom, atomid);
+        residue->SetHetAtom(atom, hetflags[i]);
+        residue->SetSerialNum(atom, sernos[i]);
+      } else {
+        name    = ChainsResName[resids[i]];
 
-            residue->AddAtom(atom);
-            residue->SetAtomID(atom, atomid);
-            residue->SetHetAtom(atom, hetflags[i]);
-            residue->SetSerialNum(atom, sernos[i]);
+        residue = mol.NewResidue();
+        residue->SetName(name);
+        residue->SetNum(resnos[i]);
+        residue->SetChain(chains[i]);
 
-            resmap[resnos[i]] = residue;
-          }
+        residue->AddAtom(atom);
+        residue->SetAtomID(atom, atomid);
+        residue->SetHetAtom(atom, hetflags[i]);
+        residue->SetSerialNum(atom, sernos[i]);
+
+        resmap[chains[i]][resnos[i]] = residue;
       }
+    }
 
     if (mol.NumResidues() == 1 && nukeSingleResidue)
       mol.DeleteResidue(mol.GetResidue(0));
-    else if (mol.NumResidues() == 1
-             && (mol.GetResidue(0))->GetName() == "UNK")
+    else if (mol.NumResidues() == 1 && (mol.GetResidue(0))->GetName() == "UNK")
       mol.DeleteResidue(mol.GetResidue(0));
   }
 
@@ -977,6 +936,7 @@ namespace OpenBabel
   bool OBChainsParser::PerceiveChains(OBMol &mol, bool nukeSingleResidue)
   {
     bool result = true;
+    unsigned int idx;
 
     SetupMol(mol);
     ClearResidueInformation(mol);
@@ -988,6 +948,81 @@ namespace OpenBabel
     result = DetermineNucleicBackbone(mol)   && result;
     result = DetermineNucleicSidechains(mol) && result;
     result = DetermineHydrogens(mol)         && result;
+    
+    // Partially identified residues
+    // example: CSD in 1LWF (CYS with two Os on the S)
+    bool changed;
+    vector<pair<char,short> > invalidResidues;
+    do {
+      changed = false;
+
+      FOR_ATOMS_OF_MOL (atom, mol) {
+        idx = atom->GetIdx() - 1;
+
+	if (resids[idx] == 0) { // UNK
+	  FOR_NBORS_OF_ATOM (nbr, &*atom) {
+	    unsigned int idx2 = nbr->GetIdx() - 1;
+	    if (resids[idx2] != 0) { // !UNK
+	      resnos[idx] = resnos[idx2];
+	      resids[idx] = resids[idx2];
+	      changed = true;
+	      
+	      bool addResidue = true;
+	      for (unsigned int i = 0; i < invalidResidues.size(); ++i)
+		if ( (invalidResidues[i].first == chains[idx2]) && 
+		     (invalidResidues[i].second == resnos[idx2]) )
+                  addResidue = false;
+	      if (addResidue)
+		invalidResidues.push_back(pair<char,short>(chains[idx2], resnos[idx2]));
+	    }
+	  }
+	}
+        
+      }
+    } while (changed);
+    for (unsigned int i = 0; i < invalidResidues.size(); ++i) {
+      FOR_ATOMS_OF_MOL (atom, mol) {
+        idx = atom->GetIdx() - 1;
+        if ( (invalidResidues[i].first == chains[idx]) && 
+           (invalidResidues[i].second == resnos[idx]) ) {
+	  hetflags[idx] = true;
+	  resids[idx] = 0; // UNK
+	  atomids[idx] = -1; 
+	}
+      }
+    }
+    invalidResidues.clear();
+
+    // number the element in the ' ' chain (water, ions, ligands, ...)
+    short resno = 1;
+    FOR_ATOMS_OF_MOL (atom, mol) {
+      idx = atom->GetIdx() - 1;
+      
+      if (atom->GetHvyValence() == 0) { 
+	chains[idx] = ' ';
+	resnos[idx] = resno;
+        resno++;
+      } else { 
+        if (resids[idx] != 0) // UNK
+	  continue;
+	if (hetflags[idx])
+	  continue;
+	
+	char chain = chains[idx];
+        FOR_ATOMS_OF_MOL (b, mol) {
+	  unsigned int idx2 = b->GetIdx() - 1;
+	  if (chains[idx2] == chain && !hetflags[idx2]) {
+            hetflags[idx2] = true;
+	    chains[idx2] = ' ';
+	    resnos[idx2] = resno;
+	    resids[idx2] = 2; // LIG
+	  }
+	}
+        
+	resno++;
+      }
+
+    }
 
     SetResidueInformation(mol, nukeSingleResidue);
     CleanupMol();
@@ -1006,14 +1041,20 @@ namespace OpenBabel
   {
     OBAtom *atom;
     vector<OBAtom *>::iterator a;
-    for (atom = mol.BeginAtom(a) ; atom ; atom = mol.NextAtom(a))
-      if (!atom->IsHydrogen() && atom->GetHvyValence() == 0)
-        {
-          // find un-connected atoms (e.g., HOH oxygen atoms)
-          //  if it's not an oxygen, it's probably some ligand
-          resids[atom->GetIdx()-1]   = (atom->IsOxygen()) ? 1 : 2;
-          hetflags[atom->GetIdx()-1] = true;
-        }
+    
+    // find un-connected atoms (e.g., HOH oxygen atoms)
+    for (atom = mol.BeginAtom(a) ; atom ; atom = mol.NextAtom(a)) {
+      if (atom->IsHydrogen() || atom->GetHvyValence() != 0)
+        continue;
+
+      unsigned int idx = atom->GetIdx() - 1;
+
+      if (atom->IsOxygen()) {
+          resids[idx]   = 1;
+          hetflags[idx] = true;
+      }
+    }
+
     return true;
   }
 
@@ -1027,8 +1068,8 @@ namespace OpenBabel
     int resno;
     int count;
     int size;
-    int i,idx;
-    int numAtoms;
+    unsigned int i, idx;
+    unsigned int numAtoms;
 
     resno    = 1;
     count    = 0;
@@ -1036,44 +1077,40 @@ namespace OpenBabel
 
     OBAtom *atom;
     vector<OBAtom *>::iterator a;
-    for (atom = mol.BeginAtom(a) ; atom ; atom = mol.NextAtom(a))
-      {
-        idx = atom->GetIdx() - 1;
-        if (!hetflags[idx] && chains[idx] == ' ' && !atom->IsHydrogen())
-          {
-            size = RecurseChain(mol, idx, 'A' + count);
+    for (atom = mol.BeginAtom(a) ; atom ; atom = mol.NextAtom(a)) {
+      idx = atom->GetIdx() - 1;
+      if (!hetflags[idx] && chains[idx] == ' ' && !atom->IsHydrogen()) {
+        size = RecurseChain(mol, idx, 'A' + count);
             
-            // size = number of heavy atoms in residue chain
-            if (size < 10) // small ligand, probably
-              {
-                if (size == 1 && atom->IsOxygen())
-                  resid = 1; /* HOH */
-                else
-                  resid = 2; /* LIG */
+        // size = number of heavy atoms in residue chain
+        if (size < 4) { // small ligand, probably
+          if (size == 1 && atom->IsOxygen())
+            resid = 1; /* HOH */
+          else
+            resid = 2; /* LIG */
 
-                for (i = 0 ; i < numAtoms ; ++i)
-                  {
-                    if (chains[i] == ('A' + count))
-                      {
-                        hetflags[i] = true;
-                        resids[i]   = resid;
-                        resnos[i]   = resno;
-                        chains[i]   = ' ';
-                      }
-                  }
-                resno++;
-              }
-            else {
-              count++; // number of connected chains
-              if (count > 26) // out of chain IDs
-                break;
+          for (i = 0 ; i < numAtoms ; ++i) {
+            if (chains[i] == ('A' + count)) {
+              hetflags[i] = true;
+              resids[i]   = resid;
+              resnos[i]   = resno;
+              chains[i]   = ' ';
             }
           }
+          resno++;
+        } else {
+          count++; // number of connected chains
+          if (count > 26) // out of chain IDs
+            break;
+        }
       }
+    }
 
+   /* 
     if( count == 1 )
       for ( i = 0 ; i < numAtoms ; i++ )
         chains[i] = ' ';
+   */
 
     return true;
   }
@@ -1109,11 +1146,23 @@ namespace OpenBabel
   {
     ConstrainBackbone(mol, Peptide, MAXPEPTIDE);
 
-    int i, max = mol.NumAtoms();
+    unsigned int i, numAtoms = mol.NumAtoms();
 
+    // Cyclic peptides have no NTer (1SKI) 
+    // timvdm 30/06/08
+    bool foundNTer = false;
+    for (i = 0 ; i < numAtoms; i++)
+      if (bitmasks[i] & BitNTer)
+	foundNTer = true;
+    if (!foundNTer)
+      for (i = 0 ; i < numAtoms; i++)
+        if (bitmasks[i] & BitNAll)
+          bitmasks[i] |= BitNTer;
+
+    
     /* Order Peptide Backbone */
 
-    for ( i = 0 ; i < max ; i++ )
+    for ( i = 0 ; i < numAtoms ; i++ )
       if (atomids[i] == -1)
         {
           if( bitmasks[i] & BitNTer )
@@ -1149,8 +1198,8 @@ namespace OpenBabel
     OBAtom *na,*nb,*nc,*nd;
     OBAtom *atom, *nbr;
     bool change, result;
-    int  count;
-    int  i,idx;
+    int count;
+    unsigned int i, idx;
 
     vector<OBAtom *>::iterator a;
     vector<OBBond *>::iterator b;
@@ -1162,8 +1211,7 @@ namespace OpenBabel
         idx = atom->GetIdx() - 1;
         bitmasks[idx] = 0;
         for ( i = 0 ; i < tmax ; i++ )
-          if ( (static_cast<unsigned int>(templ[i].elem)  == atom->GetAtomicNum())
-               &&
+          if ( (static_cast<unsigned int>(templ[i].elem)  == atom->GetAtomicNum()) &&
                (static_cast<unsigned int>(templ[i].count) == atom->GetHvyValence()))
             bitmasks[idx] |= templ[i].flag;
       }
@@ -1283,7 +1331,10 @@ namespace OpenBabel
     int na,nb,nc;
     OBAtom *atom, *nbr;
     int count;
-    int j,k,idx;
+    int j,k;
+    int idx;
+    
+    j = k = 0; // ignore warning
 
     vector<OBBond *>::iterator b;
 
@@ -1755,6 +1806,7 @@ namespace OpenBabel
               atomids[idx]  = atomids[sidx];
               resids[idx]   = resids[sidx];
               resnos[idx]   = resnos[sidx];
+              chains[idx]   = chains[sidx];
             }
         }
 
@@ -1795,7 +1847,7 @@ namespace OpenBabel
     GenerateByteCodes((ByteCode**)tree, resid, 0, 0, 0 );
   }
 
-  int OBChainsParser::IdentifyElement(const char *ptr)
+  int OBChainsParser::IdentifyElement(char *ptr)
   {
     int ch;
 
@@ -2101,7 +2153,7 @@ namespace OpenBabel
 
   const char *OBChainsParser::ParseSmiles(const char *ptr, int prev)
   {
-    const char *name;
+    char *name;
     int atomid;
     int next;
     int type;
