@@ -41,7 +41,12 @@ namespace OpenBabel
     virtual const char* Description() //required
     {
       return
-        "Crystallographic Information File\n ";
+        "Crystallographic Information File\n\n"
+        "Read Options e.g. -b:\n"
+        "  v  Verbose CIF conversion\n"
+        "  s  Output single bonds only\n"
+        "  b  Disable bonding entirely\n"
+        "  B  Use bonds listed in CIF file from _geom_bond_* records (overrides option b) \n\n";
     };
 
     virtual const char* SpecificationURL()
@@ -153,6 +158,8 @@ namespace OpenBabel
     /// Extract all atomic positions. Will generate cartesian from fractional
     /// coordinates or vice-versa if only cartesian coordinates are available.
     void ExtractAtomicPositions(const bool verbose=false);
+    /// Extract listed bond distances, from _geom_bond_* loops
+    void ExtractBonds(const bool verbose=false);
     /// Generate fractional coordinates from cartesian ones for all atoms
     /// CIFData::CalcMatrices() must be called first
     void Cartesian2FractionalCoord();
@@ -206,6 +213,16 @@ namespace OpenBabel
     };
     /// Atoms, if any are found
     std::vector<CIFAtom> mvAtom;
+    /// Bond distance record
+    struct CIFBond
+    {
+      /// Label of the two bonded atoms
+      std::string mLabel1,mLabel2;
+      /// distance
+      float mDistance;
+    };
+    /// Atoms, if any are found
+    std::vector<CIFBond> mvBond;
     /// Fractionnal2Cartesian matrix
     float mOrthMatrix[3][3];
     /// Cartesian2Fractionnal matrix
@@ -254,6 +271,7 @@ namespace OpenBabel
     this->ExtractUnitCell(verbose);
     this->ExtractSpacegroup(verbose);
     this->ExtractAtomicPositions(verbose);
+    this->ExtractBonds(verbose);
   }
    
   void CIFData::ExtractUnitCell(const bool verbose)
@@ -474,7 +492,7 @@ namespace OpenBabel
         posx=loop->second.find("_atom_site_fract_x");
         posy=loop->second.find("_atom_site_fract_y");
         posz=loop->second.find("_atom_site_fract_z");
-        unsigned int nb;
+        unsigned int nb = 0;
         if( (posx!=loop->second.end()) && (posy!=loop->second.end()) && (posz!=loop->second.end()))
           {
             nb=posx->second.size();
@@ -540,7 +558,7 @@ namespace OpenBabel
             // Now be somewhat verbose
             if(verbose)
               {
-                cout << "Found "<<nb<<" atoms. Waouh !"<<endl;
+                cout << "Found "<<nb<<" atoms."<<endl;
                 for(unsigned int i=0;i<nb;++i)
                   {
                     cout<<mvAtom[i].mLabel<<" "<<mvAtom[i].mSymbol;
@@ -562,7 +580,33 @@ namespace OpenBabel
           }
       }
   }
-   
+
+  void CIFData::ExtractBonds(const bool verbose)
+  {
+    map<ci_string,string>::const_iterator positem;
+    for(map<set<ci_string>,map<ci_string,vector<string> > >::const_iterator loop=mvLoop.begin(); loop!=mvLoop.end();++loop)
+      {
+        //if(mvBond.size()>0) break;// Only allow one bond list
+        map<ci_string,vector<string> >::const_iterator poslabel1,poslabel2,posdist;
+        poslabel1=loop->second.find("_geom_bond_atom_site_label_1");
+        poslabel2=loop->second.find("_geom_bond_atom_site_label_2");
+        posdist=loop->second.find("_geom_bond_distance");
+        if( (poslabel1!=loop->second.end()) && (poslabel2!=loop->second.end()) && (posdist!=loop->second.end()))
+          {
+            if(verbose) cout<<"Found _geom_bond* record..."<<endl;
+            const unsigned long nb=poslabel1->second.size();
+            mvBond.resize(nb);
+            for(unsigned int i=0;i<nb;++i)
+              {
+                mvBond[i].mLabel1=poslabel1->second[i];
+                mvBond[i].mLabel2=poslabel2->second[i];
+                mvBond[i].mDistance=CIFNumeric2Float(posdist->second[i]);
+                if(verbose) cout<<"  d("<<mvBond[i].mLabel1<<"-"<<mvBond[i].mLabel2<<")="<<mvBond[i].mDistance<<endl;
+              }
+          }
+      }
+  }
+
   void CIFData::CalcMatrices(const bool verbose)
   {
     if(mvLatticePar.size()==0) return;//:TODO: throw error
@@ -711,7 +755,7 @@ namespace OpenBabel
   string CIFReadValue(stringstream &in,char &lastc)
   {
     bool vv=false;//very verbose ?
-    string value;
+    string value("");
     while(!isgraph(in.peek())) in.get(lastc);
     while(in.peek()=='#')
       {//discard these comments for now
@@ -720,6 +764,11 @@ namespace OpenBabel
         lastc='\r';
         while(!isgraph(in.peek())) in.get(lastc);
       }
+    if(in.peek()=='_') {
+      if (vv)
+        cout << "WARNING: Trying to read a value but found a new tag !" << endl;
+      return value;
+    }
     if(in.peek()==';')
       {//SemiColonTextField
         bool warning=!iseol(lastc);
@@ -729,11 +778,17 @@ namespace OpenBabel
         in.get(lastc);
         while(in.peek()!=';')
           {
+            if (in.peek() == '_') {
+              cout << "WARNING: Trying to read a SemiColonTextField but found a new tag !" << endl;
+              warning = true;
+              break;
+            }
             string tmp;
             getline(in,tmp);
             value+=tmp+" ";
           }
-        in.get(lastc);
+        if (!warning)
+          in.get(lastc);
         if(vv) cout<<"SemiColonTextField:"<<value<<endl;
         if(warning && !vv) cout<<"SemiColonTextField:"<<value<<endl;
         return value;
@@ -834,14 +889,16 @@ namespace OpenBabel
                 if(in.peek()=='_') break;
                 if(in.peek()=='#')
                   {// Comment (in a loop ??)
+                    const std::ios::pos_type pos=in.tellg();
                     string tmp;
                     getline(in,tmp);
                     if(block=="") mvComment.push_back(tmp);
                     else mvData[block].mvComment.push_back(tmp);
                     lastc='\r';
                     if(vv) cout<<"Comment in a loop (?):"<<tmp<<endl;
-                    continue;
-                  };
+                    in.seekg(pos);
+                    break;
+                  }
                 const std::ios::pos_type pos=in.tellg();
                 in>>tmp;
                 if(vv) cout<<"WHATNEXT? "<<tmp;
@@ -898,20 +955,109 @@ namespace OpenBabel
     return v;
   }
 
-  //############################## END CIF CLASSES####################################################
+  //################ END CIF CLASSES######################################
 
   //Make an instance of the format class
   CIFFormat theCIFFormat;
 
+  // Helper function for CorrectFormatCharges
+  // Is this atom an oxygen in a water molecule
+  // We know the oxygen is connected to one ion, but check for non-hydrogens
+  // Returns: true if the atom is an oxygen and connected to two hydrogens and up to one other atom
+  bool isWaterOxygen(OBAtom *atom)
+  {
+    if (!atom->IsOxygen())
+      return false;
+     
+    int nonHydrogenCount = 0;
+    int hydrogenCount = 0;
+    FOR_NBORS_OF_ATOM(neighbor, *atom) {
+      if (!neighbor->IsHydrogen())
+        nonHydrogenCount++;
+      else
+        hydrogenCount++;
+    }
+    
+    return (hydrogenCount == 2 && nonHydrogenCount <= 1);
+  }
+
+  // Look for lone ions, and correct their formal charges
+  void CorrectFormalCharges(OBMol *mol)
+  {
+    if (!mol)
+      return;
+    
+    // First look for NR4, PR4 ions,
+    // or bare halides, alkali and alkaline earth metal ions
+    FOR_ATOMS_OF_MOL(atom, *mol) {
+      
+      if ((atom->GetAtomicNum() == 7 || atom->GetAtomicNum() == 15) 
+          && atom->BOSum() == 4) {
+        // check if we should make a positive charge?
+        // i.e., 4 non-metal neighbors
+        bool nonMetalNeighbors = true;
+        FOR_NBORS_OF_ATOM(neighbor, &*atom) {
+          switch (neighbor->GetAtomicNum()) {
+          case 1:
+          case 5: case 6: case 7: case 8: case 9:
+          case 14: case 15: case 16: case 17:
+          case 33: case 34: case 35:
+          case 53:
+            continue; // good non-metals
+          default:
+            nonMetalNeighbors = false;
+            break; // stop looking
+          }
+        }
+        if (nonMetalNeighbors) // 4 non-metals, e.g. NH4+
+          atom->SetFormalCharge(+1);
+      }
+      
+      // Now look for simple atomic ions like Na, Li, F, Cl, Br...
+      // If we have an existing formal charge, keep going
+      if (atom->GetFormalCharge() != 0)
+        continue;
+
+      // If we're connected to anything besides H2O, keep going
+      if (atom->GetValence() != 0) {
+        int nonWaterBonds = 0;
+        FOR_NBORS_OF_ATOM(neighbor, &*atom) {
+          if (!isWaterOxygen(&*neighbor)) {
+            nonWaterBonds = 1;
+            break;
+          }
+        }
+        if (nonWaterBonds)
+          continue; // look at another atom
+      }
+
+      switch(atom->GetAtomicNum()) {
+      case 3: case 11: case 19: case 37: case 55: case 87:
+        // Alkali ions
+        atom->SetFormalCharge(+1);
+        break;
+      case 4: case 12: case 20: case 38: case 56: case 88:
+        // Alkaline earth ions
+        atom->SetFormalCharge(+2);
+        break;
+      case 9: case 17: case 35: case 53: case 85:
+        // Halides
+        atom->SetFormalCharge(-1);
+        break;
+      }
+    }
+  }
+
   /////////////////////////////////////////////////////////////////
   bool CIFFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
   {
     OBMol* pmol = dynamic_cast<OBMol*>(pOb);
     if(pmol==NULL)
       return false;
+    bool verbose=false;
+    if (pConv->IsOption("v",OBConversion::INOPTIONS)) verbose=true;
 
-    CIF cif(*pConv->GetInStream(),true,false);
-
+    CIF cif(*pConv->GetInStream(),true,verbose);
     // Loop on all data blocks until we find one structure :TODO: handle multiple structures
     for(map<string,CIFData>::iterator pos=cif.mvData.begin();pos!=cif.mvData.end();++pos)
       if(pos->second.mvAtom.size()>0)
@@ -941,16 +1087,19 @@ namespace OpenBabel
 
           if(pos->second.mFormula!="") pmol->SetFormula(pos->second.mFormula);
 
+          // Keep a map linking the cif atom label to the obatom*, for bond interpretation later
+          std::map<std::string,OBAtom *> vLabelOBatom;
+          
           const unsigned int nbatoms=pos->second.mvAtom.size();
           pmol->ReserveAtoms(nbatoms);
           for(vector<CIFData::CIFAtom>::const_iterator posat=pos->second.mvAtom.begin();posat!=pos->second.mvAtom.end();++posat)
             {
-              // Problem: posat->mSymbol is not garanteed to actually be a symbol
+              // Problem: posat->mSymbol is not guaranteed to actually be a symbol
               // see http://www.iucr.org/iucr-top/cif/cifdic_html/1/cif_core.dic/Iatom_type_symbol.html
               // Try to strip the string to have a better chance to have a valid symbol
               // This is not guaranteed to work still, as the CIF standard allows about any string...
               string tmpSymbol=posat->mSymbol;
-              int nbc=0;
+              unsigned int nbc=0;
               if((tmpSymbol.size()==1) && isalpha(tmpSymbol[0])) nbc=1;
               else if(tmpSymbol.size()>=2)
                 {
@@ -960,6 +1109,8 @@ namespace OpenBabel
               
               OBAtom *atom  = pmol->NewAtom();
               
+              vLabelOBatom.insert(make_pair(posat->mLabel,atom));
+              
               if(tmpSymbol.size()>nbc)
                 {// Try to find a formal charge in the symbol
                   int charge=0;
@@ -988,11 +1139,37 @@ namespace OpenBabel
             }
           if (!pConv->IsOption("b",OBConversion::INOPTIONS))
             pmol->ConnectTheDots();
+          if (pConv->IsOption("B",OBConversion::INOPTIONS))
+            {
+              for(vector<CIFData::CIFBond>::const_iterator posbond=pos->second.mvBond.begin();posbond!=pos->second.mvBond.end();++posbond)
+                {// Add bonds present in the cif and not detected by ConnectTheDots()
+                  std::map<std::string,OBAtom *>::iterator posat1,posat2;
+                  posat1=vLabelOBatom.find(posbond->mLabel1);
+                  posat2=vLabelOBatom.find(posbond->mLabel2);
+                  if(posat1!=vLabelOBatom.end() && posat2!=vLabelOBatom.end())
+                    {
+                      if(verbose) cout<<"  Adding cif bond ? "<<posat1->first<<"-"<<posat2->first;
+                      if(pmol->GetBond(posat1->second,posat2->second)==NULL)
+                        {
+                           if(verbose) cout<<"  :Bond added !"<<endl;
+                           OBBond * bond=pmol->NewBond();
+                           bond->SetBegin(posat1->second);
+                           bond->SetEnd(posat2->second);
+                           bond->SetBondOrder(1);
+                           bond->SetLength(double(posbond->mDistance));
+                        }
+                       else if(verbose) cout<<"  :Bond already present.. "<<endl;
+                    }
+                }
+            }
           if (!pConv->IsOption("s",OBConversion::INOPTIONS) && !pConv->IsOption("b",OBConversion::INOPTIONS))
             pmol->PerceiveBondOrders();
           pmol->EndModify();
+          pmol->SetAutomaticFormalCharge(false); // we should have set formal charges
+          CorrectFormalCharges(pmol); // Look for lone Na -> Na+, etc.
           return true;
         }
+
     // If we got here, no structure was found
     obErrorLog.ThrowError(__FUNCTION__, "Problems reading a CIF file: no structure found !" , obWarning);
     return(false);
