@@ -76,6 +76,10 @@ namespace OpenBabel
     vector<double> charges;
     bool hasPartialCharges = false;
     double energy;
+    OBVectorData *dipoleMoment = NULL;
+    bool readingVibrations = false;
+    vector< vector<vector3> > displacements; // vibrational displacements
+    vector<double> frequencies, intensities;
 
     mol.BeginModify();
     while	(ifs.getline(buffer,BUFF_SIZE))
@@ -130,6 +134,97 @@ namespace OpenBabel
                 tokenize(vs,buffer);
                 if (vs.size() < 1) vs.push_back(string()); // timvdm 18/06/2008
               }
+            // Now we should be at DIPOLE line
+            ifs.getline(buffer,BUFF_SIZE);	// POINT CHARGE
+            ifs.getline(buffer,BUFF_SIZE);	// HYBRID
+            ifs.getline(buffer,BUFF_SIZE);	// SUM
+            tokenize(vs, buffer);
+            if (vs.size() == 5) {
+              if (dipoleMoment)
+                delete dipoleMoment;
+
+              dipoleMoment = new OBVectorData;
+              double x, y, z;
+              x = atof(vs[1].c_str());
+              y = atof(vs[2].c_str());
+              z = atof(vs[3].c_str());
+              dipoleMoment->SetData(x, y, z);
+              dipoleMoment->SetAttribute("Dipole Moment");
+              dipoleMoment->SetOrigin(fileformatInput);
+            }
+
+            if (!ifs.getline(buffer,BUFF_SIZE))
+              break;
+          }
+        else if(strstr(buffer,"MASS-WEIGHTED COORDINATE ANALYSIS") != NULL)
+          { // the correct vibrations -- earlier bits aren't mass-weighted
+            readingVibrations = true;
+            if (!ifs.getline(buffer,BUFF_SIZE))
+              break;
+          }
+        else if (readingVibrations && strstr(buffer, "Root No.") != NULL)
+          {
+            ifs.getline(buffer, BUFF_SIZE); // blank line
+            ifs.getline(buffer, BUFF_SIZE); // symmetry labels (for OB-2.3)
+            ifs.getline(buffer, BUFF_SIZE); // blank
+            ifs.getline(buffer, BUFF_SIZE); // frequencies
+            tokenize(vs, buffer);
+            for (unsigned int i = 0; i < vs.size(); ++i) {
+              frequencies.push_back(atof(vs[i].c_str()));
+            }
+            ifs.getline(buffer, BUFF_SIZE); // blank
+
+            // now real work
+            int prevModeCount = displacements.size();
+            int newModes = frequencies.size() - displacements.size();
+            vector<vector3> displacement;
+            for (unsigned int i = 0; i < newModes; ++i) {
+              displacements.push_back(displacement);
+            } 
+
+            ifs.getline(buffer, BUFF_SIZE);
+            tokenize(vs, buffer);
+            int modeCount = vs.size();
+            vector<double> x, y, z;
+            while(modeCount > 1) {
+              x.clear();
+              for (unsigned int i = 1; i < modeCount; ++i) {
+                x.push_back(atof(vs[i].c_str()));
+              }
+              y.clear();
+              ifs.getline(buffer, BUFF_SIZE);
+              tokenize(vs, buffer);
+              for (unsigned int i = 1; i < modeCount; ++i) {
+                y.push_back(atof(vs[i].c_str()));
+              }
+              
+              z.clear();
+              ifs.getline(buffer, BUFF_SIZE);
+              tokenize(vs, buffer);
+              for (unsigned int i = 1; i < modeCount; ++i) {
+                z.push_back(atof(vs[i].c_str()));
+              }
+
+              // OK, now we have x, y, z for all new modes for one atom
+              for (unsigned int i = 0; i < modeCount - 1;  ++i) {
+                displacements[prevModeCount + i].push_back(vector3(x[i], y[i], z[i]));
+              }
+
+              // Next set of atoms
+              ifs.getline(buffer, BUFF_SIZE);
+              tokenize(vs, buffer);
+              modeCount = vs.size();
+            }
+          }
+        else if (readingVibrations && strstr(buffer, "T-DIPOLE") != NULL)
+          {
+            unsigned int currentIntensity = intensities.size();
+            tokenize(vs, buffer);
+            if (vs.size() < 2)
+              break;
+
+            double transDipole = atof(vs[1].c_str());
+            intensities.push_back(frequencies[currentIntensity] * transDipole * transDipole);
           }
       }
 
@@ -140,7 +235,8 @@ namespace OpenBabel
 
     if (!pConv->IsOption("b",OBConversion::INOPTIONS))
       mol.ConnectTheDots();
-    if (!pConv->IsOption("s",OBConversion::INOPTIONS) && !pConv->IsOption("b",OBConversion::INOPTIONS))
+    if (!pConv->IsOption("s",OBConversion::INOPTIONS) 
+        && !pConv->IsOption("b",OBConversion::INOPTIONS))
       mol.PerceiveBondOrders();
 
     mol.EndModify();
@@ -156,9 +252,19 @@ namespace OpenBabel
         OBPairData *dp = new OBPairData;
         dp->SetAttribute("PartialCharges");
         dp->SetValue("Mulliken");
-        dp->SetOrigin(perceived);
+        dp->SetOrigin(fileformatInput);
         mol.SetData(dp);
       }
+    if (dipoleMoment)
+      mol.SetData(dipoleMoment);
+    if (frequencies.size() != 0) { // we found some vibrations
+      OBVibrationData *vd = new OBVibrationData;
+      vd->SetData(displacements, frequencies, intensities);
+      vd->SetOrigin(fileformatInput);
+      mol.SetData(vd);
+    }
+
+
     mol.SetTitle(title);
 
     return(true);
